import java.awt.*;
import java.util.Vector;

/** 
 ** HFillLayout is a layout manager used for stacking components
 ** horizontally. It is not a flow layout - it recognises components
 ** which may be arbitrarily large, i.e. Canvas, Panel and TextArea,
 ** and expands them to fill the vertical and horizontal space
 ** available. A flow layout would allocate to these components only
 ** their minimum size, which for Canvas is zero height and zero width,
 ** and leave a blank space to the right of the components. If there are
 ** expandable components, HFillLayout will use the entire space.
 ** @author Friendless (friendless@usa.net)
 **/
public class HFillLayout implements LayoutManager {
  int hgap, vgap;
  int vfill;
  Vector exceptions;

  /** 
   ** <B>CENTRE</B> makes a component its preferred height and centres the
   ** component.<BR>
   **/
  public static final int CENTRE = 0;
  /** 
   ** <B>CENTER</B> makes a component its preferred height and centres the
   ** component. This alternate spelling included for foreigners.<BR>
   **/
  public static final int CENTER = 0;
  /**
   ** <B>TOP</B> makes a component its preferred size and places it at
   ** the top.<BR>
   **/
  public static final int TOP = 1; 
  /**
   ** <B>BOTTOM</B> makes a component its preferred size and places it
   ** at the bottom.<BR>
   **/
  public static final int BOTTOM = 2;
  /**
   ** <B>FILL</B> makes the component as high as the entire row.
   **/
  public static final int FILL = 3;

  /**
   ** By default, components are centred vertically within the row.
   **/
  public HFillLayout() {
    this(CENTRE,4,4);
    }

  /**
   ** @param vfill specifies how the components are placed horizontally
   ** @param hgap specifies the vertical distance between components.
   ** @param vgap specifies the horizontal distance components are inset from the side.
   ** within the column. See CENTRE, TOP, BOTTOM, FILL above.
   **/
  public HFillLayout(int vfill, int hgap, int vgap) {
    this.vfill = vfill;
    this.hgap = hgap;
    this.vgap = vgap;
    exceptions = new Vector(5);
    }

  /**
   ** Inform the HFillLayout that this component should not be expanded.
   ** Does not cause a repaint or relayout.
   **/
  public void addException(Component c) {
    exceptions.addElement(c);
    }

  /**
   ** Inform the HFillLayout that this component should now be expanded if 
   ** it remains in the layout.
   ** Does not cause a repaint or relayout.
   **/
  public boolean removeException(Component c) {
    return exceptions.removeElement(c);
    }

  /** If you add a component with a name, it will not be expanded. **/
  public void addLayoutComponent(String name, Component comp) {
    addException(comp);
    }

  private boolean expandable(Component c) {
    if (exceptions.contains(c)) return false;
    if (c instanceof Canvas) return true;
      else if (c instanceof Panel) return true;
      else if (c instanceof TextArea) return true;
      else if (c instanceof TextField) return true;
      else return false;
    }

  /** Relocates the components of parent according to this layout. **/
  public void layoutContainer(Container parent) {
    boolean min;
    int height, width, zeroCount, expansion, h, v;
    Rectangle b = parent.bounds();
    width = 0;
    height = 0;
    zeroCount = 0;
    Component[] cs = parent.getComponents();
    // calculate preferred sizes
    for (int i=0; i<cs.length; i++) {
        Dimension d = cs[i].preferredSize();
        if (expandable(cs[i])) {
            zeroCount++;
            }
          else {
            width += d.width;
            }
        width += hgap;
        }
    width -= hgap;
    // calculate size to expand zero height components to
    if (width >= b.width) {
        // preferred size is too big, so try minimum size
        min = true;
        width = 0;
        for (int i=0; i<cs.length; i++) {
            Dimension d = cs[i].minimumSize();
            if (!expandable(cs[i])) width += d.width;
            width += hgap;
            }
        width -= hgap;
        expansion = 0;
        if ((zeroCount != 0) && (width < b.width)) {
            expansion = (b.width - width) / zeroCount;
            }
        }
      else if (zeroCount == 0) {
        // no expandable components
        min = false;
        expansion = 0;
        }
      else {
        min = false;
        expansion = (b.width-width)/zeroCount;
        }
    // layout
    width = 0;
    for (int i=0; i<cs.length; i++) {
        Dimension d = null;
        h = 0;
        // figure out vertical size
        if (min) d = cs[i].minimumSize();
          else d = cs[i].preferredSize();
        // only a TextField expands horizontally but not vertically
        if (cs[i] instanceof TextField) {
            h = expansion;
            v = d.height;
            }
          else if (expandable(cs[i])) {
            h = expansion;
            v = 0;
            }
          else {
            h = d.width;
            v = d.height;
            }
        // figure out horizontal placement
        int top = vgap;
        int maxheight = b.height;
        if (vfill == FILL) v = maxheight;
          else if ((v == 0) || (v > maxheight)) v = maxheight;
        if (vfill == BOTTOM) top = b.height - vgap - v;
          else if (vfill == CENTER) top = top + (maxheight-v)/2;
        // place it
        cs[i].reshape(width,top,h,v);
        width += h;
        width += hgap;
        }
    }

  /** Returns the minimum size on which this component may be drawn. **/
  public Dimension minimumLayoutSize(Container parent) {
    int height, width;
    height = 0;
    width = hgap;
    Component[] cs = parent.getComponents();
    for (int i=0; i<cs.length; i++) {
        Dimension d = cs[i].minimumSize();
        width += d.width;
        width += hgap;
        if (d.height > height) height = d.height;
        }
    return new Dimension(width - hgap,height);
    }

  /** Returns the preferred size that this container would be if all
   ** components told the truth about their size. Canvases are notorious
   ** liars.
   **/
  public Dimension preferredLayoutSize(Container parent) {
    int height, width;
    height = 0;
    width = hgap;
    Component[] cs = parent.getComponents();
    for (int i=0; i<cs.length; i++) {
        Dimension d = cs[i].preferredSize();
        width += d.width;
        width += hgap;
        if (d.height > height) height = d.height;
        }
    return new Dimension(width-hgap,height);
    }

  /** This only removes components from the exception list. **/
  public void removeLayoutComponent(Component comp) {
    removeException(comp);
    }

  /** Returns a summary of this object as a string. **/
  public String toString() {
    return getClass().getName() + "[hgap=" + hgap + 
				       ",vfill=" + vfill + "]";
    }
}
