import java.awt.*;

/** This is a component on which you can place an image.
 ** I've extended it so you can add a second image. When the image is clicked on,
 ** it's replaced by the second image for 1/5 of a second, and an event is passed
 ** to a designated event handler, so you can use it as an image button.
 ** Use JDK 1.02 event model.
 **
 ** @author Friendless - friendless@usa.net
 **/
public class ImageCanvas extends Canvas implements Runnable {
  Image img, img2;
  Component eventHandler;
  MediaTracker tracker;
  boolean onImage2;

  protected void init() {
    tracker = new MediaTracker(this);
    img = null;
    img2 = null;
    eventHandler = null;
    onImage2 = false;
    }

  /** Set no images or event handler. **/
  public ImageCanvas() {
    this(null,null,null);
    }

  /** Set only an image. **/
  public ImageCanvas(Image i) {
    this(i,null,null);
    }

  /** Set two images and an event handler.
   ** @param i the image which is usually displayed.
   ** @param i2 the image which is displayed after a click
   ** @param handler the component which will handle the click event
   **/
  public ImageCanvas(Image i, Image i2, Component handler) {
    init();
    img = i;
    img2 = i2;
    if (i != null) tracker.addImage(i,0);
    if (i2 != null) tracker.addImage(i2,0);
    eventHandler = handler;
    }

  /** Set the first image. **/
  public void setImage(Image i) {
    img = i;
    tracker.addImage(i,0);
    repaint(100);
    }

  /** Set both images. **/
  public void setImage(Image i, Image i2) {
    img = i;
    img2 = i2;
    tracker.addImage(i,0);
    tracker.addImage(i2,0);
    repaint(100);
    }

  /** Set the event handler. **/
  public void setEventHandler(Component handler) {
    eventHandler = handler;
    }

  /** Preferred size is the size of the first image.
   **/
  public Dimension preferredSize() {
    try {
        tracker.waitForID(0);
        }
    catch (InterruptedException ie) {
        return super.preferredSize();
        }
    if (img == null) return super.preferredSize();
      else return new Dimension(img.getWidth(this),img.getHeight(this));
    }

  /** Minimum size is also the size of the preferred image.
   **/
  public Dimension minimumSize() {
    try {
        tracker.waitForID(0);
        }
    catch (InterruptedException ie) {
        return super.minimumSize();
        }
    if (img == null) return super.minimumSize();
      else return preferredSize();
    }

  public void update(Graphics g) { paint(g); }

  /** paint() uses double buffering to avoid flicker.
   ** The unused area around the image is filled with the background colour.
   **/
  public void paint(Graphics g) {
    if (getPeer() == null) return;
    Dimension sz = size();
    Image off = createImage(sz.width,sz.height);
    Graphics goff = off.getGraphics();
    goff.setColor(getBackground());
    goff.fillRect(0,0,sz.width,sz.height);
    try {
        tracker.waitForID(0);
        }
    catch (InterruptedException ie) {
        return;
        }
    Image i = img;
    if (onImage2 && (img2 != null)) i = img2;
    if (i != null) {
        int xoff = i.getWidth(this);
        if (sz.width > xoff) xoff = (sz.width-xoff)/2;
          else xoff = 0;
        int yoff = i.getHeight(this);
        if (sz.height > yoff) yoff = (sz.height-yoff)/2;
          else yoff = 0;
        goff.drawImage(i,xoff,yoff,this);
        }
    g.drawImage(off,0,0,this);
    }

  public boolean mouseDown(Event event, int x, int y) {
    if (eventHandler == null) return false;
      else {
        onImage2 = true;
        repaint();
        Thread t = new Thread(this);
        t.start();
        return eventHandler.mouseDown(event,x,y);
        }
    }

  /** This object spawns a thread after a mouse click to wait and set the image
   ** back to the original.
   **/
  public void run() {
    try {
        Thread.sleep(200);
        }
    catch (InterruptedException ie) {
        }
    onImage2 = false;
    repaint();
    }
}
